.tour-wrap{padding:20px;}.tour-wrap .hidden{visibility:hidden}.tour-wrap .tour-name{margin:0 0 24px 0;font-weight:bold;font-size:16px}.tour-wrap .tour-bar-wrap{position:relative;height:20px;}.tour-wrap .tour-bar-wrap .tour-bar{background-color:#d9dde2;height:5px;font-size:1px;overflow:hidden;position:relative;top:-3px;}.tour-wrap .tour-bar-wrap .tour-bar .completed{width:0;height:100%;background-color:#534472}.tour-wrap .tour-bar-wrap a.step-circle{position:absolute;top:-6.5px;color:#ccc;-webkit-border-radius:50%;border-radius:50%;width:13px;height:13px;background-color:#d9dde2;margin-left:-6.5px;}.tour-wrap .tour-bar-wrap a.step-circle.available:hover{border:3px solid #4195c2;margin-left:-9.5px;top:-9.5px}.tour-wrap .tour-bar-wrap a.step-circle.complete:hover,.tour-wrap .tour-bar-wrap a.step-circle.current:hover{border:3px solid #534472;margin-left:-9.5px;top:-9.5px}.tour-wrap .tour-bar-wrap a.step-circle.current.available{background-color:#534472}.tour-wrap .tour-bar-wrap a.step-circle.incomplete.unavailable{cursor:default}.tour-wrap .tour-bar-wrap a.step-circle.complete{background-color:#534472}.tour-wrap .tour-bar-wrap a.step-circle .step-name{visibility:hidden;position:absolute;top:-43px;padding:10px;-webkit-border-radius:6px;border-radius:6px;background-color:#4195c2;color:#fff;white-space:nowrap;font-family:'HalisRLight',helvetica,arial,verdana,sans-serif;text-transform:uppercase;font-size:10px;font-weight:bold;}.tour-wrap .tour-bar-wrap a.step-circle .step-name:after{top:100%;left:50%;border:solid transparent;content:" ";height:0;width:0;position:absolute;pointer-events:none;background-color:transparent;border-top-color:#4195c2;border-width:6px;margin-left:-6px}.tour-wrap .tour-bar-wrap a.step-circle:hover .step-name{visibility:visible}.tour-wrap .tour-bar-wrap a.step-circle:first-child{margin-left:0;}.tour-wrap .tour-bar-wrap a.step-circle:first-child.available:hover,.tour-wrap .tour-bar-wrap a.step-circle:first-child.complete:hover,.tour-wrap .tour-bar-wrap a.step-circle:first-child.current:hover{margin-left:0}.tour-wrap .tour-bar-wrap a.step-circle:first-child .step-name{left:0;-webkit-border-radius:6px 6px 6px 0;border-radius:6px 6px 6px 0;}.tour-wrap .tour-bar-wrap a.step-circle:first-child .step-name:after{left:0;margin-left:0}.tour-wrap .tour-bar-wrap a.step-circle:last-child{margin-left:-13px;}.tour-wrap .tour-bar-wrap a.step-circle:last-child.available:hover,.tour-wrap .tour-bar-wrap a.step-circle:last-child.complete:hover,.tour-wrap .tour-bar-wrap a.step-circle:last-child.current:hover{margin-left:-19px}.tour-wrap .tour-bar-wrap a.step-circle:last-child .step-name{right:0;-webkit-border-radius:6px 6px 0 6px;border-radius:6px 6px 0 6px;}.tour-wrap .tour-bar-wrap a.step-circle:last-child .step-name:after{right:0;left:auto;margin-left:0}