{% if tour %}
    <div class="tour-wrap">
        <div class="tour-name">
            {{ tour.display_name }}
        </div>
        <div class="tour-bar-wrap hidden">
            <div class="tour-bar">
                <div class="completed"></div>
            </div>
            <span class="tour-step-wrap">
                {% for step in tour.steps %}
                    <a href="{{ step.url }}" class="step-circle {{ step.cls }}">
                        <span class="step-name">{{ forloop.counter }}. {{ step.name }}</span>
                    </a>
                {% endfor %}
            </span>
        </div>
    </div>
    <link rel="stylesheet" href="{{ STATIC_URL }}tour/build/tour.css" />
    <script src="{{ STATIC_URL }}tour/build/tour.js"></script>
    <script>
        (new window.DjangoTour()).run();
    </script>
{% endif %}
