!function(){"use strict";window.DjangoTour=function(){},window.DjangoTour.prototype={positionTourElements:function(a){var b=a.getElementsByClassName("step-circle"),c=a.getElementsByClassName("step-name"),d=0,e=b.length;if(0!==e){var f=0;e>1&&(f=100/(e-1));for(var g=null,h=0;e>h;h++){if(b[h].style.left=f*h+"%",d=b[h].offsetWidth,h>0&&e-1>h){var i=-(c[h].offsetWidth/2)+b[h].offsetWidth/2;c[h].style.marginLeft=i+"px"}for(var j={},k=b[h].className.split(" "),l=0;l<k.length;l++)j[k[l]]=!0;"current"in j&&"available"in j?g=b[h]:"incomplete"in j&&"available"in j&&h>0&&!g&&(g=b[h])}var m=a.getElementsByClassName("completed")[0],n=0;g&&(n=parseFloat(g.style.left),m.style.width=n+"%");var o=a.getElementsByClassName("tour-bar-wrap")[0];o.className=o.className.replace("hidden","")}},run:function(){for(var a=document.getElementsByClassName("tour-wrap"),b=0;b<a.length;b++)this.positionTourElements(a[b])}}}();